@@ -32,10 +32,11 @@
 #include <canberra-gtk.h>
 
 #include "gvc-channel-bar.h"
+#include "gvc-mixer-control.h"
 
 #define SCALE_SIZE 128
-#define ADJUSTMENT_MAX_NORMAL 65536.0 /* PA_VOLUME_NORM */
-#define ADJUSTMENT_MAX_AMPLIFIED 98304.0 /* 1.5 * ADJUSTMENT_MAX_NORMAL */
+#define ADJUSTMENT_MAX_NORMAL gvc_mixer_control_get_vol_max_norm(NULL)
+#define ADJUSTMENT_MAX_AMPLIFIED gvc_mixer_control_get_vol_max_amplified(NULL)
 #define ADJUSTMENT_MAX (bar->priv->is_amplified ? ADJUSTMENT_MAX_AMPLIFIED : ADJUSTMENT_MAX_NORMAL)
 #define SCROLLSTEP (ADJUSTMENT_MAX / 100.0 * 5.0)
 
@@ -315,7 +316,7 @@ gvc_channel_bar_set_low_icon_name   (GvcChannelBar *bar,
                 bar->priv->low_icon_name = g_strdup (name);
                 gtk_image_set_from_icon_name (GTK_IMAGE (bar->priv->low_image),
                                               bar->priv->low_icon_name,
-                                              GTK_ICON_SIZE_BUTTON);
+                                              GTK_ICON_SIZE_MENU);
                 g_object_notify (G_OBJECT (bar), "low-icon-name");
         }
 }
@@ -331,7 +332,7 @@ gvc_channel_bar_set_high_icon_name  (GvcChannelBar *bar,
                 bar->priv->high_icon_name = g_strdup (name);
                 gtk_image_set_from_icon_name (GTK_IMAGE (bar->priv->high_image),
                                               bar->priv->high_icon_name,
-                                              GTK_ICON_SIZE_BUTTON);
+                                              GTK_ICON_SIZE_MENU);
                 g_object_notify (G_OBJECT (bar), "high-icon-name");
         }
 }
@@ -449,7 +450,7 @@ gvc_channel_bar_scroll (GvcChannelBar *bar, GdkScrollDirection direction)
                 /* Switch side scroll to vertical */
                 if (direction == GDK_SCROLL_RIGHT)
                         direction = GDK_SCROLL_UP;
-                else if (GDK_SCROLL_LEFT)
+                else if (direction == GDK_SCROLL_LEFT)
                         direction = GDK_SCROLL_DOWN;
         }
 
@@ -616,8 +617,8 @@ gvc_channel_bar_set_is_amplified (GvcChannelBar *bar, gboolean amplified)
 
                 g_free (str);
                 gtk_alignment_set (GTK_ALIGNMENT (bar->priv->mute_box), 0.5, 0, 0, 0);
-                gtk_misc_set_alignment (GTK_MISC (bar->priv->low_image), 0.5, 0);
-                gtk_misc_set_alignment (GTK_MISC (bar->priv->high_image), 0.5, 0);
+                gtk_misc_set_alignment (GTK_MISC (bar->priv->low_image), 0.5, 0.15);
+                gtk_misc_set_alignment (GTK_MISC (bar->priv->high_image), 0.5, 0.15);
                 gtk_misc_set_alignment (GTK_MISC (bar->priv->label), 0, 0);
         } else {
                 gtk_alignment_set (GTK_ALIGNMENT (bar->priv->mute_box), 0.5, 0.5, 0, 0);
@@ -907,10 +908,10 @@ gvc_channel_bar_init (GvcChannelBar *bar)
         gtk_container_add (GTK_CONTAINER (bar->priv->mute_box), bar->priv->mute_button);
 
         bar->priv->low_image = gtk_image_new_from_icon_name ("audio-volume-low-symbolic",
-                                                             GTK_ICON_SIZE_BUTTON);
+                                                             GTK_ICON_SIZE_MENU);
         gtk_widget_set_no_show_all (bar->priv->low_image, TRUE);
         bar->priv->high_image = gtk_image_new_from_icon_name ("audio-volume-high-symbolic",
-                                                              GTK_ICON_SIZE_BUTTON);
+                                                              GTK_ICON_SIZE_MENU);
         gtk_widget_set_no_show_all (bar->priv->high_image, TRUE);
 
         bar->priv->image = gtk_image_new ();
